function nowAsISOString()
  return os.date("!%Y-%m-%dT%H:%M:%S")
end

function sleep(n)
  os.execute("sleep " .. tonumber(n))
end

-- split a string with delimiters into a table (reverse of table.concat)
function split(s, delim)
  assert(type(delim) == "string" and string.len(delim) > 0, "bad delimiter")

  local start = 1
  local t = {} -- results table

  -- find each instance of a string followed by the delimiter

  while true do
    local pos = string.find(s, delim, start, true) -- plain find

    if not pos then
      break
    end

    table.insert(t, string.sub(s, start, pos - 1))
    start = pos + string.len(delim)
  end -- while

  -- insert final one (after last delimiter)

  table.insert(t, string.sub(s, start))

  return t
end

-- trim leading and trailing spaces from a string
function trim(s)
  return (string.gsub(s, "^%s*(.-)%s*$", "%1"))
end

-- print table to string
function table2String(t, indent)
  local names = {}
  if not indent then
    indent = ""
  end
  for n, g in pairs(t) do
    table.insert(names, n)
  end
  table.sort(names)

  local res = ""
  for i, n in pairs(names) do
    local v = t[n]
    if type(v) == "table" then
      if (v == t) then -- prevent endless loop if table contains reference to itself
        res = res .. indent .. tostring(n) .. ": <-" .. "\n"
      else
        res = res .. indent .. tostring(n) .. ":" .. "\n"
        res = res .. table2String(v, indent .. "   ")
      end
    else
      if type(v) == "function" then
        res = res .. indent .. tostring(n) .. "()" .. "\n"
      else
        res = res .. indent .. tostring(n) .. ": " .. tostring(v) .. "\n"
      end
    end
  end
  return res
end

-- url encode and decode
function encodeURIComponent(str)
  if (str) then
    str = string.gsub(str, "\n", "\r\n")
    str =
      string.gsub(
      str,
      "([^%w ])",
      function(c)
        return string.format("%%%02X", string.byte(c))
      end
    )
    str = string.gsub(str, " ", "+")
  end
  return str
end

function decodeURIComponent(s)
  return (string.gsub(
    string.gsub(s, "+", " "),
    "%%(%x%x)",
    function(str)
      return string.char(tonumber(str, 16))
    end
  ))
end

-- parse querystring to table
function parseQueryString(str, sep, eq)
  if not sep then
    sep = "&"
  end
  if not eq then
    eq = "="
  end
  local vars = {}
  for pair in string.gmatch(tostring(str), "[^" .. sep .. "]+") do
    if not string.find(pair, eq) then
      vars[decodeURIComponent(pair)] = ""
    else
      local key, value =
        string.match(pair, "([^" .. eq .. "]*)" .. eq .. "(.*)")
      if key then
        vars[decodeURIComponent(key)] = decodeURIComponent(value)
      end
    end
  end
  return vars
end

-- table to querystring
function table2Qs(t, sep, eq, ne)
  if not sep then
    sep = "&"
  end
  if not eq then
    eq = "="
  end
  if not ne then
    ne = false
  end

  local tmp = {}
  for k, v in pairs(t) do
    if not ne then
      table.insert(tmp, encodeURIComponent(k) .. eq .. encodeURIComponent(v))
    else
      table.insert(tmp, k .. eq .. "'" .. v .. "'")
    end
  end

  return table.concat(tmp, sep)
end

-- render utils
function fileExists(file)
  local f = io.open(file, "rb")
  if f then
    f:close()
  end
  return f ~= nil
end

function readFileAsText(file)
  if not fileExists(file) then
    return {}
  end
  lines = {}
  for line in io.lines(file) do
    lines[#lines + 1] = line
  end
  return lines
end

-- render
function httpStatus(code)
  if code ~= nil then
    print("Status: " .. code)
  end
end

function renderString(str, options)
  local lines = str
  if type(str) == "string" then
    lines = split(str, "\n")
  end
  local v
  for _, v in pairs(lines) do
    for s1, s2 in pairs(options) do
      v =
        string.gsub(
        v,
        "<%%[-=]%s*" .. s1 .. "%s*%%>",
        function()
          return s2
        end
      )
    end
    print(v)
  end
end

function renderFile(f, options)
  local lines = readFileAsText(f)
  renderString(lines, options)
end

function renderHtml(f, options)
  httpStatus(options.status)
  print("Content-Type: text/html")
  print("Connection: close")

  if options then
    if options.mac then
      print("Set-Cookie: Device-MAC=" .. options.mac .. "; path=/")
      print("Set-Cookie: client_mac=" .. options.mac .. "; path=/")
    end
  end
  print("")
  print("")
  renderFile(f, options)
end

-- response
function response(str, options)
  httpStatus(options.status)
  if options then
    if options.type then
      print("Content-Type: " .. options.type)
    else
      print("Content-Type: text/plain")
    end
    if options.mac then
      print(
        "Set-Cookie: Device-MAC=" ..
          options.mac .. "; expires= Tue, 31 Dec 2999 23:59:59 GTM; path=/"
      )
    end
  else
    print("Content-Type: text/html")
  end
  print("Connection: Close")
  print("Access-Control-Allow-Origin: *")
  print("")

  print(str)
end

-- redirect
function redirect(url)
  httpStatus()
  print("Content-Type: text/html")
  print("Connection: Close")
  print("")
  print("<!DOCTYPE html>")
  print("<html>")
  print("<head>")
  print('<meta http-equiv="refresh" content="0;URL=\'' .. url .. '\'"/>')
  print('<meta charset="utf-8">')
  print('<meta http-equiv="x-ua-compatible" content="ie=edge">')
  print(
    '<meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">'
  )
  print("<title>Redirecting...</title>")
  print("</head>")
  print("<body>")
  print("Redirecting...")
  print("<script>")
  print("window.location.href = '" .. url .. "';")
  print("</script>")
  print("<body>")
  print("</html>")
end
